/*
 *  Copyright (c) 2001 Rudolf Heeb <Rudolf.Heeb@gmx.de>
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 */

#include <math.h>

#include <qpainter.h>
#include <qxembed.h>
#include <qpopupmenu.h>


#include <klocale.h>
#include <kglobal.h>
#include <kapp.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>

extern "C" {
#include <X11/Xlib.h>
#include <X11/XKBlib.h>
}



#include "kled.h"
#include "kled.moc"

extern "C"
{
    KPanelApplet* init(QWidget *parent, const QString& configFile)
    {
	KGlobal::locale()->insertCatalogue("kledapplet");
        KledApplet *applet = new KledApplet(configFile, KPanelApplet::Normal, 
                                            KPanelApplet::About, parent, "kledapplet");
        return applet;
    }
}

KledApplet::KledApplet(const QString& configFile, Type t, int actions,
                       QWidget *parent, const char *name)
  : KPanelApplet( configFile, t, actions, parent, name ), _aboutData(0)
{
    display = kapp->getDisplay();
    disp=this->x11Display();
    root = RootWindow(display,DefaultScreen(display));

    startTimer(127); 
}

int KledApplet::widthForHeight(int h) const
{
    return static_cast<int>(1.4 * h); // rectangular shape.
}
int KledApplet::heightForWidth(int w) const
{
    return static_cast<int>(w / 1.4); // rectangular shape.
}

void KledApplet::resizeEvent( QResizeEvent*e )
{
    QWidget::resizeEvent(e);
}

void KledApplet::timerEvent(QTimerEvent*)
{
    // draw pupils
    QPainter p(this);
    drawLeds(&p);
}

void KledApplet::about()
{
    if(!_aboutData) {
        _aboutData = new KAboutData("kledapplet", I18N_NOOP("KLedApplet"), "0.1",
                                    I18N_NOOP("Keyboard LED indicators applet.\n\n"
                                  "The goal is to show you your state of CAPS- SCROLL- and NUM-lock of your Keyboard.\n"),
                                    KAboutData::License_BSD, "(c) 2001, Rudolf Heeb");
        _aboutData->addAuthor("Rudolf Heeb", 0, "Rudolf.Heeb@gmx.de");
    }

    KAboutApplication dialog(_aboutData);
    dialog.show();
}


void KledApplet::paintEvent(QPaintEvent* e)
{
    // draw background
    QWidget::paintEvent(e);

    // draw LEDS
    QPainter p(this);
    drawLeds(&p);

}

void KledApplet::drawLeds(QPainter* p)
{
    int i,bit;
    int w = width()-1;
    int h = height()-1;
    int py = 0;
    int hi = (h/2);
    int wi = (w/3);
    unsigned int states;

    static const char *fonts[] = { "Helvetica", "Courier", "Times", 0 };
    static int   sizes[] = { 10, 12, 18, 24, 36, 0 };
    QFont font( fonts[0], sizes[1] );
    p->setFont( font );
    p->setPen(QPen(black, 2));
 
    XkbGetIndicatorState(disp,XkbUseCoreKbd,&states); 

    for (i=0,bit=1;i<XkbNumIndicators;i++,bit<<=1) 
    {
        int flag;
        flag=states&bit;
        switch (i) 
        {
         case 1: // Numlock
           if (flag)
             p->setBrush(QBrush(green));
           else
             p->setBrush(QBrush(white));
           p->drawRect(0, py, wi, hi);
           break;
         case 0: // CapsLock
           if (flag)
             p->setBrush(QBrush(green));
           else
             p->setBrush(QBrush(white));
      	 p->drawRect(w/3, py, wi, hi);
           break;
         case 2: //ScrollLock
           if (flag)
             p->setBrush(QBrush(green));
           else
             p->setBrush(QBrush(white));
           p->drawRect(((w/3)*2),py, wi, hi);
           break;
        }
    }
    p->drawText( 0+2,h, "N" );
    p->drawText( (w/3)+2,h, "C" );
    p->drawText( ((w/3)*2)+2,h, "S" );

}

